/**************************************************************************
 * Client-Server communication using TCP/IP (AF_INET sockets)
 * This is client
 *
 * program will send message to the server. Port is compiled in.
 * Example: ./tcpc server.com "hi"
 *
 * 1. make an communication endpoint - socket()
 * 2. connection to the socket - connect()
 * 3. write data to the socket - write()
 * 4. read data from socket - read()
****************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>

#define PORT 9734		/* port, where server waits for connection */

int
main (int argc, char *argv[])
{
  int sockfd, result, len;
  struct sockaddr_in address;
  struct hostent *he;

  if (argc != 3)
    {
      fprintf (stderr, "Usage: %s host message\n", argv[0]);
      exit (1);
    }

  if ((he = gethostbyname (argv[1])) == NULL)
    {
      perror ("I'm not able to get information about host");
      exit (1);
    }

  if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
    {
      perror ("Error in socket creation");
      exit (1);
    }

  address.sin_family = AF_INET;
  address.sin_addr = *((struct in_addr *) he->h_addr);
  address.sin_port = htons (PORT);
  bzero (&(address.sin_zero), 8);	// the rest of structure will be zero


  /* print the host IP in "dot notation" */
  printf ("client IP: %s\n", inet_ntoa (address.sin_addr));

  result = connect (sockfd, (struct sockaddr *) &address, sizeof (address));

  if (result == -1)
    {
      perror ("I'm not able to connect to server");
      exit (1);
    }

  len = strlen (argv[2]);
  printf ("client: sending message %s to host \"%s\" %d bytes long\n", argv[2], argv[1], len);
  if ((result = send (sockfd, argv[2], len, 0)) == -1)
    {
      perror ("message sending succesful");
      exit (1);
    }
  else
    {
      printf ("client: %d bytes of message has been sent\n", result);
    }
  close (sockfd);
  exit (0);
}
