/*****************************************************************************
 * Client-Server communication using TCP/IP (AF_INET sockets)
 * This is server
 * 
 * 1. make an communication endpoint - socket()
 * 2. bind socket to data structure - bind()
 * 3. set the connection acceptance and max. num of acceptable connections - listen()
 * 4. wait for connection - accept()
 *
 * Remark: use ^C to terminate the server
 ****************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <unistd.h>

#define MYPORT 9734		/* port, where I'll listen for connection */
#define BACKLOG 5		/* max. num of accepted connections */
#define MAXBUFLEN 100

int
main (void)
{
  int server_sockfd, client_sockfd;
  int client_len, len, i;
  struct sockaddr_in addr_s, addr_c;
  char buf[MAXBUFLEN];

  printf("press ^C to terminate\n");
  if ((server_sockfd = socket (AF_INET, SOCK_STREAM, 0)) == -1)	/* TCP */
    {
      perror ("socket creation error");
      exit (1);
    }

  addr_s.sin_family = AF_INET;
  addr_s.sin_addr.s_addr = INADDR_ANY;
  addr_s.sin_port = htons (MYPORT);
  bzero (&(addr_s.sin_zero), 8);

  if (bind (server_sockfd, (struct sockaddr *) &addr_s,
	    sizeof (addr_s)) == -1)
    {
      perror ("bind error");
      exit (1);
    }

  if (listen (server_sockfd, BACKLOG) == -1)
    {
      perror ("listen error");
      exit (1);
    }

  i = 5;
  while (i)
    {
      printf ("server: waiting\n");
      client_len = sizeof (struct sockaddr);
      /* accept() creates new descriptor */
      if ((client_sockfd =
	   accept (server_sockfd, (struct sockaddr *) &addr_c,
		   &client_len)) == -1)
	{
	  perror ("accept error");
	  exit (1);
	}
      if ((len = recv (client_sockfd, buf, MAXBUFLEN - 1,0)) == -1)
	{
	  perror ("Error receiving message");
	  exit (1);
	}
      printf ("packet from host %s", inet_ntoa (addr_c.sin_addr));
      printf (" has %d bytes\n", len);
      buf[len] = '\0';
      printf ("packet content: \"%s\"\n", buf);
      close (client_sockfd);
      i--;
    }
  close(server_sockfd);
  return 0;
}
